<?php

/**
 * @file
 * fee administration menu items.
 * Author: Bobby Kramer, Marketplace Port Of Original Ubercart Fee Module http://drupal.org/project/uc_fee , author:  mrfelton
 *	2011-01-23
*/

// Builds a paged list and overview of existing product fees.
function mp_fee_admin_form() {
  
  $header = _mp_fee_admin_header();
  $form = array();

  //FIXME: Do we need to limit the columns we ask for?
  if(user_access('administer all fees')){
  	$result = pager_query("SELECT  f.fid, f.name, f.weight, f.type, f.price, f.is_active FROM {mp_fees} AS f GROUP BY f.fid, f.name, f.weight, f.type, f.price". tablesort_sql($header), 30, 0, "SELECT COUNT(fid) FROM {mp_fees}");
  }else if(user_access('administer own fees')){
  	global $user;
  	//query to get all non-universal fees associated with this user
  	$result = pager_query("SELECT  f.fid, f.name, f.weight, f.type, f.price, f.is_active FROM {mp_fees} AS f WHERE f.uid = %d and f.is_universal = 0 GROUP BY f.fid, f.name, f.weight, f.type, f.price". tablesort_sql($header), 30, 0, "SELECT COUNT(fid) FROM {mp_fees}", $user->uid);
  	
  }
  while ($fee = db_fetch_object($result)) {
    $form['fees'][$fee->fid] = array(
      'name' => array(
        '#value' => $fee->name,
      ),
      'weight' => array(
        '#type' => 'weight',
        '#default_value' => $fee->weight,
        '#attributes' => array('class' => 'uc-fee-table-weight'),
      ),
      'type' => array(
        '#value' => theme('mp_fee_type', $fee),
      ),
      'price' => array(
        '#value' => theme('mp_fee_price', $fee),
      ),
      'is_active' => array(
        '#value' => ($fee->is_active)? 'yes' : 'no',
      ),
      'ops' => array(
        '#value' => l(t('edit'), 'store/fees/'. $fee->fid .'/edit') .' '.
                    l(t('delete'), 'store/fees/'. $fee->fid .'/delete') .' ',
                    
      ),
    );
  }

  if (!empty($form['fees'])) {
    $form['fees']['#tree'] = TRUE;

    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save changes'),
      '#weight' => 10,
    );
  }

  return $form;
}

// Displays a paged list and overview of existing product fees.
function theme_mp_fee_admin_form($form) {
  $header = _mp_fee_admin_header();

  if (count(element_children($form['fees'])) > 0) {
    foreach (element_children($form['fees']) as $fid) {
      $row = array(
        drupal_render($form['fees'][$fid]['name']),
        drupal_render($form['fees'][$fid]['weight']),
        drupal_render($form['fees'][$fid]['type']),
        drupal_render($form['fees'][$fid]['price']),
        drupal_render($form['fees'][$fid]['is_active']),
        drupal_render($form['fees'][$fid]['ops']),
      );

      $rows[] = array(
        'data' => $row,
        'class' => 'draggable',
      );
    }
  }
  else {
    $rows[] = array(
      array('data' => t('No fees have been added yet.'), 'colspan' => 5),
    );
  }

  drupal_add_tabledrag('uc-fee-table', 'order', 'sibling', 'uc-fee-table-weight');
  $output = theme('table', $header, $rows, array('id' => 'uc-fee-table'));
  $output .= drupal_render($form);

  return $output;
}

function mp_fee_admin_form_submit($form, &$form_state) {

  foreach ($form_state['values']['fees'] as $fid => $fee) {
    db_query("UPDATE {mp_fees} SET weight = %d WHERE fid = %d", $fee['weight'], $fid);
  }
  drupal_set_message(t('The changes have been saved.'));
}

function _mp_fee_admin_header() {
  return array(
    array('data' => t('Name'), 'field' => 'f.name'),
    array('data' => t('Order'), 'field' => 'f.weight', 'sort' => 'asc'),
    array('data' => t('Type'), 'field' => 'f.type'),
    array('data' => t('Price'), 'field' => 'f.price'),
    array('data' => t('Active'), 'field' => 'f.is_active'),
    t('Operations'),
  );
}


/**
 * Form builder for product fees.
 *
 * @ingroup forms
 * @see mp_fee_form_validate()
 * @see mp_fee_form_submit()
 */
function mp_fee_form($form_state, $fee = NULL) {
  // If a fee is specified, add its ID as a hidden value.
  global $user;
    //if there is a $fee, we know it's on the edit form.
  if (!empty($fee)) {  	
  	//TODO:Should probbaly be moved to access arguements somehow
    //The problem is, you need to have the user and the fee
  	//make sure the proper user permissions, since the access call does not cover all cases	
    if(user_access('administer all fees')){  	
    	//do nothing, their the admin, continue
  	}else if(user_access('administer own fees') ){
  		//make sure the fee owner matches the current user trying to access the edit form
  		if($fee->uid != $user->uid){
  		  	return;
  		}
  	}else{
  		//return, bad permissions
  		return;
  	}
    $form['fid'] = array('#type' => 'hidden', '#value' => $fee->fid);
    drupal_set_title(t('Edit fee: %name', array('%name' => $fee->name)));
  }

  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#description' => t('This name will appear to customers on product add to cart forms.'),
    '#default_value' => $fee->name,
    '#required' => TRUE,
  );
  $form['description'] = array(
    '#type' => 'textfield',
    '#title' => t('Description'),
    '#description' => t('<b>Optional.</b> Enter the description that will display beneath the fee in the checkout and order panes.'),
    '#default_value' => $fee->description,
    '#maxlength' => 255,
  );
  $form['price'] = array(
    '#type' => 'textfield',
    '#title' => t('Price'),
    '#default_value' => $fee->price,
    '#description' => t("The amount as a currency valur or percentage, depending on the value of 'Type' (above)."),
    '#size' => 6,
    '#required' => TRUE,
  );
  
  $form['is_active'] = array(
    '#title' => t('Active'),
    '#type' => 'checkbox',
    '#description' => 'If not active, this fee will not be applied regardless of any conditions or product level settings.',
    '#default_value' => isset($fee->is_active)? $fee->is_active : 1,
  );
  //display these ONLY if user has permission to administer site-wide fees
   if(user_access('administer all fees')){
   	  $form['type'] = array(
    	'#type' => 'select',
    	'#title' => t('Type'),
    	'#description' => t("This determines how the fee will be applied. Possible options are:") .'<br/><b>'. t("Absolute:") .'</b>'. t(" The fee is an absolute price and will be added to the base product price as is.") .'<br/><b>'. t("Percentage:") .'</b>'. t(" The fee works as a percentage multiplier on the base price of the product eg. '10' = 10% (base_price x 1.1)"),
    	'#options' => array(MP_FEE_TYPE_ABSOLUTE => t('Absolute'), MP_FEE_TYPE_PERCENTAGE => t('Percentage')),
    	'#default_value' => $fee->type? $fee->type : MP_FEE_TYPE_ABSOLUTE,
  	);    
 	 $form['weight'] = array(
    	'#title' => t('Weight'),
    	'#type' => 'weight',
    	'#description' => t('Determines the order in which fees are applied.'),
    	'#required' => TRUE,
    	'#default_value' => isset($fee->weight)? $fee->weight : 0,
 	 );

  
 	$form['is_universal'] = array(
    	'#title' => t('Universal'),
    	'#type' => 'checkbox',
    	'#description' => t('If checked, the fee will be applied across all sellers, and sellers have no control over it\'s presence'),
    	'#default_value' => $fee->is_universal,
  	);
  }//end user_access if statement (only for admins)
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
    '#suffix' => l(t('Cancel'), 'store/fees/' . $user->uid),
  );

  return $form;
}

/**
 * Submit function for mp_fee_add_form().
 */
function mp_fee_form_submit($form, &$form_state) {
	global $user;
  if (!empty($form_state['values']['fid'])) {
    db_query("UPDATE {mp_fees} SET name = '%s', description = '%s',  price = %f, type = %d,  weight = %d, is_active = %d,  is_universal = %d WHERE fid = %d",
	     $form_state['values']['name'],
	     $form_state['values']['description'],
	     $form_state['values']['price'],
	     ($form_state['values']['type'] ? $form_state['values']['type'] : MP_FEE_TYPE_ABSOLUTE),
	     ($form_state['values']['weight'] ? $form_state['values']['weight'] : -11),
	     $form_state['values']['is_active'],
	     ($form_state['values']['is_universal'] ? $form_state['values']['is_universal'] : 0 ),
	     $form_state['values']['fid']
	     );

  }
  else {
  	
    db_query("INSERT INTO {mp_fees} (uid, name, description, price, type,  weight, is_active, is_universal) VALUES (%d, '%s', '%s', %f, %d, %d, %d, %d)",
		$user->uid,
      $form_state['values']['name'],
      $form_state['values']['description'],
      $form_state['values']['price'],
      ($form_state['values']['type'] ? $form_state['values']['type'] : MP_FEE_TYPE_ABSOLUTE),
      ($form_state['values']['weight'] ? $form_state['values']['weight'] : -11),
      $form_state['values']['is_active'],
      ($form_state['values']['is_universal'] ? $form_state['values']['is_universal'] : 0 )
      );
  }

  $form_state['redirect'] = 'store/fees/' . $user->uid;
}

// Confirms the deletion of the given fee.
function mp_fee_delete_confirm($form_state, $fee) {
	global $user;
  // If we got a bunk fee, kick out an error messag\e.
  if (empty($fee)) {
  	
    drupal_set_message(t('There is no fee with that ID.'), 'error');
    drupal_goto('store/fees/' . $user->uid);
  }
  //TODO:Should probbaly be moved to access arguements somehow
  //The problem is, you need to have the user and the fee
  	//make sure the proper user permissions, since the access call does not cover all cases	
    if(user_access('administer all fees')){  	
    	//do nothing, their the admin, continue
  	}else if(user_access('administer own fees') ){
  		//make sure the fee owner matches the current user trying to access the edit form
  		if($fee->uid != $user->uid){
  		  	return;
  		}
  	}else{
  		//return, bad permissions
  		return;
  	}

  $form['fid'] = array('#type' => 'value', '#value' => $fee->fid);
  $form['#redirect'] = 'store/fees/' . $user->uid;
  $output = confirm_form($form, t('Are you sure you want to delete the fee %name?', array('%name' => $fee->name)),
              'store/fees/' . $user->uid, '',
              t('Delete'), t('Cancel'));

  return $output;
}

function mp_fee_delete_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    db_query("DELETE FROM {mp_fees} WHERE fid = %d", $form_state['values']['fid']);
    drupal_set_message(t('fee deleted.'));
  }
}





